--------------------------------------------------------------------------------
-- SubModule FIFOPipeOut
-- Created   8/28/2006 2:16:56 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;

entity FIFOPipeOut is port
   (
     ti_clk            : in    std_logic;
     ti_control        : in    std_logic_vector(12 downto 0);
     ti_data           : inout   std_logic_vector(15 downto 0);
     ep_status         : out   std_logic_vector(3 downto 0);
     ep_reset          : in    std_logic;
     ep_clk            : in    std_logic;
     ep_write          : in    std_logic;
     ep_full           : out   std_logic;
     ep_empty          : out   std_logic;
     ep_error          : out   std_logic;
     ep_addr           : in    std_logic_vector(7 downto 0);
     ep_datain         : in    std_logic_vector(31 downto 0);
     ep_enable         : in    std_logic
   );
end FIFOPipeOut;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of FIFOPipeOut is

-- Component Declarations
    component okPipeOut port (
        ti_clk      : in    std_logic;
        ti_control  : in    std_logic_vector(12 downto 0);
        ti_data     : inout   std_logic_vector(15 downto 0);
        ep_addr     : in    std_logic_vector(7 downto 0);
        ep_datain   : in    std_logic_vector(15 downto 0);
        ep_read     : out   std_logic);
    end component;

    component fifo_write port (
        din         : in    std_logic_vector(31 downto 0);
        rd_clk      : in    std_logic;
        rd_en       : in    std_logic;
        rst         : in    std_logic;
        wr_clk      : in    std_logic;
        wr_en       : in    std_logic;
        dout        : out   std_logic_vector(15 downto 0);
        empty       : out   std_logic;
        full        : out   std_logic;
        overflow    : out   std_logic;
        underflow   : out   std_logic;
        wr_data_count: out  std_logic_vector(3 downto 0));
    end component;

-- Signal Declarations
   signal data             : std_logic_vector(15 downto 0);
   signal rden             : std_logic;
   signal wren             : std_logic;
   signal overflow         : std_logic;
   signal underflow        : std_logic;

begin
   rden <= ep_write when ep_enable = '1' else '0';

   pipe : okPipeOut port map (
      ti_clk => ti_clk,
      ti_control => ti_control,
      ti_data => ti_data,
      ep_addr => ep_addr,
      ep_datain => data,
      ep_read => wren);

   fifo : fifo_write port map (
      wr_clk => ep_clk,
      wr_en => rden,
      din => ep_datain,
      rd_clk => ti_clk,
      rd_en => wren,
      dout => data,
      rst => ep_reset,
      empty => ep_empty,
      full => ep_full,
      wr_data_count => ep_status,
      overflow => overflow,
      underflow => underflow);

   ep_error <= '0' when ep_reset = '1' else
               '1' when overflow = '1' or underflow = '1';

end Structure;
--------------------------------------------------------------------------------
