--------------------------------------------------------------------------------
-- SubModule IIR_ADC4
-- Created   4/12/2005 5:51:13 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity IIR_ADC4 is port
   (
     ADCA       : in    std_logic_vector(17 downto 0);  -- AD7679
     ADCB       : in    std_logic_vector(17 downto 0);  -- AD7679
     ADCC       : in    std_logic_vector(17 downto 0);  -- AD7679
     ADCD       : in    std_logic_vector(17 downto 0);  -- AD7679
     L          : in    std_logic;  -- busy signal of ADC
     O          : out   std_logic_vector(31 downto 0);  -- sign extended ADC value
     OVF        : out   std_logic   -- overflow
   );
end IIR_ADC4;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of IIR_ADC4 is

   signal extA : std_logic_vector(31 downto 0);
   signal extB : std_logic_vector(31 downto 0);
   signal extC : std_logic_vector(31 downto 0);
   signal extD : std_logic_vector(31 downto 0);
   signal sum1 : signed(31 downto 0) := (others => '0');
   signal sum2 : signed(31 downto 0) := (others => '0');
   signal diff : signed(31 downto 0) := (others => '0');
   signal reg : std_logic_vector(31 downto 0) := (others => '0');

begin
   -- sign extend and shift ADC value into 32 bit word
   load: process (L) is
   begin
      if rising_edge(L) then
         extA(6 downto 0) <= (others => '0');
         extA(24 downto 7) <= ADCA;
         extA(31 downto 25) <= (others => ADCA(17));
         extB(6 downto 0) <= (others => '0');
         extB(24 downto 7) <= ADCB;
         extB(31 downto 25) <= (others => ADCB(17));
         extC(6 downto 0) <= (others => '0');
         extC(24 downto 7) <= ADCC;
         extC(31 downto 25) <= (others => ADCC(17));
         extD(6 downto 0) <= (others => '0');
         extD(24 downto 7) <= ADCD;
         extD(31 downto 25) <= (others => ADCD(17));
      end if;
   end process load;
   sum1 <= signed(extA) + signed(extC);
   sum2 <= signed(extB) + signed(extD);
   diff <= sum1 - sum2;
   -- load new ADC value into register
   O <= std_logic_vector(diff);

   -- check overflow on ADC input
   overflow: process (L) is
   begin
      if rising_edge (L) then
         if (ADCA(17 downto 6) = B"011111111111") or
            (ADCA(17 downto 6) = B"100000000000") or
            (ADCB(17 downto 6) = B"011111111111") or
            (ADCB(17 downto 6) = B"100000000000") or
            (ADCC(17 downto 6) = B"011111111111") or
            (ADCC(17 downto 6) = B"100000000000") or
            (ADCD(17 downto 6) = B"011111111111") or
            (ADCD(17 downto 6) = B"100000000000") then
            OVF <= '1';
         else
            OVF <= '0';
         end if;
      end if;
   end process overflow;

end Structure;
--------------------------------------------------------------------------------
