
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.std_logic_unsigned.all;


entity okHostInterface is
    port (
        hi_clk      : in std_logic;
        hi_rdwr     : in std_logic;
        hi_cs       : in std_logic;
        hi_addr     : in std_logic_vector(3 downto 0);
        hi_data     : inout std_logic_vector(15 downto 0);
        hi_irq      : out std_logic;
        hi_busy     : out std_logic;
        ti_clk      : out std_logic;
        ti_control  : out std_logic_vector(12 downto 0);
        ti_data     : inout std_logic_vector(15 downto 0)
    );
end okHostInterface;

architecture arch of okHostInterface is
    component BUFG port (
        I : in std_logic;
        O : out std_logic);
    end component;

    component IOBUF port (
        T : in std_logic;
        O : out std_logic;
        I : in std_logic;
        IO : inout std_logic);
    end component;

    component okHostInterfaceCore port (
        hi_clk : in std_logic;
        hi_rdwr : in std_logic;
        hi_cs : in std_logic;
        hi_irq : out std_logic;
        hi_busy : out std_logic;
        hi_dir : out std_logic;
        hi_addr : in std_logic_vector(3 downto 0);
        hi_datain : in std_logic_vector(15 downto 0);
        hi_dataout : out std_logic_vector(15 downto 0);
        ti_control : out std_logic_vector(12 downto 0);
        ti_data : inout std_logic_vector(15 downto 0));
    end component;

    signal hi_datain : std_logic_vector(15 downto 0);
    signal hi_dataout : std_logic_vector(15 downto 0);
    signal hi_dir : std_logic;
    signal ti_clk_int : std_logic;
begin

ti_clk <= ti_clk_int;

-- Clock buffer for the Host Interface clock.
--clkbuf : BUFG port map (I => hi_clk, O => ti_clk_int);
ti_clk_int <= hi_clk;

-- Instantiate bidirectional IOBUFs for the hi_data lines.
iobuf_gen : for i in 0 to 15 generate
begin
   buf  : IOBUF port map (T => hi_dir, O => hi_datain(i),  I => hi_dataout(i),  IO => hi_data(i) );
end generate iobuf_gen;

-- Instantiate the core Host Interface.
hicore : okHostInterfaceCore port map(
    hi_clk => ti_clk_int,
    hi_rdwr => hi_rdwr,
    hi_cs => hi_cs,
    hi_irq => hi_irq,
    hi_busy => hi_busy,
    hi_addr => hi_addr,
    hi_dir => hi_dir,
    hi_datain => hi_datain,
    hi_dataout => hi_dataout,
    ti_control => ti_control,
    ti_data => ti_data);

end arch;


